import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.*;
import java.awt.image.*;
import java.awt.Color;

public class Bomb extends ExactLocated {
    public int timeToExplosion; // TODO: upgradeable?
    private static final GreenfootImageCutter sprite = Resources.mastervirSprite;
    private static GreenfootImage[] frames;

    private Hitbox hitbox = new Hitbox(24, 24, this);
    private Player kickedByPlayer;

    private int range;

    private Direction dir = Direction.DOWN;
    private boolean moving = false;
    private float speed = 0.6f;

    public Bomb(int range, int timeToExplosion) {
        this.range = range;
        this.timeToExplosion = timeToExplosion;
        this.setImage(frames[0]);
    }

    public static void loadFrames() {
        frames = getFreshFrames();
    }

    public static GreenfootImage[] getFreshFrames() {
        GreenfootImage[] frames = new GreenfootImage[3];
        for (int i = 0; i < 3; i++) {
            frames[i] = sprite.getSubImage(520 + 22 * i + (i == 2 ? 1 : 0), 145, 20, 24);
        }

        for (GreenfootImage image : frames) {
            ImageUtils.replaceColor(image, Color.WHITE, new Color(0, 0, 0, 0));
            image.scale((int) (20 * 1.65), (int) (24 * 1.65));
        }

        return frames;
    }

    public static GreenfootImage[] getFrames() {
        return frames;
    }

    @Override
    public void firstAct() {
        moveToFieldCenter();
        setExactFromOfficial();
        getWorld().addObject(hitbox, getX(), getY());

        Sounds.get("csgo_bomb_planted", 80).play();
    }

    @Override
    public void doAct() {
        if (ticksAlive >= timeToExplosion) {
            explode();
            return;
        }

        if (moving) {
            float exactXForeseen = exactX + dir.getModX()*speed;
            float exactYForeseen = exactY + dir.getModY()*speed;

            hitbox.setLocation((int) exactXForeseen,(int) exactYForeseen);

            for (Actor a : hitbox.getIntersections())
                if (a instanceof LocalPlayer || a instanceof OnlinePlayer || a instanceof Block || a instanceof DestroyableBlock || (a instanceof Bomb && !a.equals(this))){
                    System.out.println("Hit " + a);
                    kickedByPlayer = null;
                    moving = false;
                    break;
                } 
            if (moving){
                setLocationExact(exactXForeseen, exactYForeseen);
            }
        }

        if (ticksAlive % 17 != 0) return; 
        this.setImage(frames[ticksAlive % 2 + 1]);
    }

    public void explode() {
        getWorld().addObject(new Explosion(range), getX(), getY());
        Sounds.get("boom", 50).play();
        getWorld().removeObject(hitbox);
        getWorld().removeObject(this);
    }

    public void kick(Direction dir, float speed, Player kickedByPlayer){
        System.out.println("kick");
        if (!kickedByPlayer.equals(this.kickedByPlayer)) {
            moving = true;
            this.kickedByPlayer = kickedByPlayer;
            this.dir = dir;
            this.speed = speed;
        }
    }
}
