import greenfoot.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Collections;
import java.net.Socket;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * Write a description of class GameHost here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class GameHost
{
    private ConcurrentHashMap<Integer, Actor> actorsOnMap = new ConcurrentHashMap<Integer, Actor>();
    private ConcurrentHashMap<Socket, SocketStreams> streamsOfSockets = new ConcurrentHashMap<Socket, SocketStreams>();
    private SocketInfo[] info;          //info and clients are in same order
    private ArrayList<Socket> clients;
    private int currentID = 0; // should be incremented after every call
    
    private int threadsFinishedTracker = 0;
    
    public GameHost(MapTemplate mapTemplate, ArrayList<Socket> clients){
        this.clients = clients;
        this.info = new SocketInfo[clients.size()];
        
        for (int i = 0; i < clients.size(); i++){
            final int temp = i;
            new Thread(() ->
            {
                int playerposition = temp;
                try{
                    ObjectOutputStream oos = new ObjectOutputStream(clients.get(playerposition).getOutputStream());
                    ObjectInputStream ois = new ObjectInputStream(clients.get(playerposition).getInputStream());
                    
                    streamsOfSockets.put(clients.get(playerposition), new SocketStreams(ois, oos));
                    
                    oos.writeInt(playerposition); // give client his player's position
                    oos.flush();
                                
                    int chosenPlayer = ois.readInt(); // get the chosen player
                                
                    //String playername = dis.readString("playername");
                                
                    info[playerposition] = (new SocketInfo(chosenPlayer,playerposition));
                    threadsFinishedTracker++;
                }catch(Exception e){e.printStackTrace();}
            }).start();
        }
        
        //waiting until everybody has given own information
        
        while (threadsFinishedTracker != clients.size()){
            try{
                Thread.sleep(100);
            }catch(Exception e){e.printStackTrace();}
        }
        
        //everyone has given own information to the host
        
        for (int y = 0, playerindex = 0; y < mapTemplate.levelTemplate.length; y++)
            for (int x = 0; x < mapTemplate.levelTemplate[y].length; x++){
                if (mapTemplate.levelTemplate[y][x] == 'b'){
                    actorsOnMap.put(new Integer(currentID),new Block(mapTemplate.style));
                }else if (mapTemplate.levelTemplate[y][x] == 'd'){
                    actorsOnMap.put(new Integer(currentID),new DestroyableBlock(mapTemplate.style));
                }else if (mapTemplate.levelTemplate[y][x] == 'p'){
                    if (!(playerindex + 1 > clients.size())){
                        actorsOnMap.put(new Integer(currentID), new OnlinePlayer(info[playerindex].chosenPlayer));
                        playerindex++;
                    }
                }
                currentID++;
            }
            
        for (int i = 0; i < clients.size(); i++){
            final int temp = i;
            new Thread(() ->
            {
                int index = temp;
                try{
                    ObjectOutputStream oos = streamsOfSockets.get(clients.get(index)).oos;  
                    
                                        
                    oos.writeInt(info[index].chosenPlayer);
                    oos.flush();
                    
                    //Thread.sleep(20);
                    
                    oos.writeInt(clients.size());
                    oos.flush();
                    
                    //Thread.sleep(20);
                    
                    oos.writeObject(mapTemplate);
                    oos.flush();
                    
                    ///oos.close();
                                
                }catch(Exception e){e.printStackTrace();}
            }).start();
        }
        
        for (int i = 0; i < clients.size(); i++){
            ObjectOutputStream[] clientsToSendTo = new ObjectOutputStream[clients.size()-1];
            ObjectInputStream objectsFromPlayer = null;
            
            int indexInArray = 0;
            for (int o = 0; o < clients.size(); o++){
                if (!clients.get(0).equals(clients.get(o))){
                    clientsToSendTo[indexInArray] = streamsOfSockets.get(clients.get(o)).oos;
                    indexInArray++;
                }else{
                    objectsFromPlayer = streamsOfSockets.get(clients.get(o)).ois;
                }
            }
            new Thread(new RecieveAndShare(clientsToSendTo, objectsFromPlayer)).start();
        }
    }
    
    public synchronized void doProtocol(Object protocol, ObjectOutputStream[] clientsToSendTo){
        try{
            if (protocol instanceof PlayerUpdateProtocol){
                System.out.println("GameHost: recieved PlayerUpdateProtocol");
                for (ObjectOutputStream oos : clientsToSendTo){
                    oos.writeObject(protocol);
                    oos.flush();
                }
                System.out.println("GameHost: sent it to all other clients");
            }else if (protocol instanceof NewBombProtocol){
                
            }else if (protocol instanceof BombUpdateProtocol){
                
            }else if (protocol instanceof BlockDestroyedProtocol){
                
            }else if (protocol instanceof PlayerDiedProtocol){
                
            }
        }catch(Exception e){e.printStackTrace();}
    }
    
    private class SocketInfo{
        public int chosenPlayer;
        public int position;
        //public String name;
        public SocketInfo(int chosenPlayer, int position){
            this.chosenPlayer = chosenPlayer;
            this.position = position;
        }
    }
    
    private class SocketStreams{
        public ObjectInputStream ois;
        public ObjectOutputStream oos;
        public SocketStreams(ObjectInputStream ois, ObjectOutputStream oos){
            this.ois = ois;
            this.oos = oos;
        }
    }
    
    private class RecieveAndShare implements Runnable{
        ObjectOutputStream[] otherClients;
        ObjectInputStream objectsFromPlayer;
        
        public RecieveAndShare(ObjectOutputStream[] otherClients, ObjectInputStream objectsFromPlayer){
            this.otherClients = otherClients;
            this.objectsFromPlayer = objectsFromPlayer;
        }
        
        public void run(){
            Object recievedObject;
            try{
                while ((recievedObject = objectsFromPlayer.readObject()) != null){
                    doProtocol(recievedObject, otherClients);
                }
            }catch(Exception e){e.printStackTrace();}
        }
    }
}
