import greenfoot.*;
import java.awt.*;
import java.awt.image.*;
import java.util.Arrays;

/**
 * A class to cut greenfoot images.
 */
public class GreenfootImageCutter {

    private GreenfootImage image;
    
    public GreenfootImageCutter(String path) {
        this(new GreenfootImage(path));
    }
    
    public GreenfootImageCutter(GreenfootImage image) {
        this.image = image;
    }
    
    public GreenfootImage getSubImage(int x, int y, int width, int height) {
        GreenfootImage result = new GreenfootImage(width, height);
        
        BufferedImage tmp = image.getAwtImage().getSubimage(x, y, width, height);
        Graphics2D graphics = (Graphics2D) result.getAwtImage().getGraphics();
        graphics.drawImage(tmp, null, 0, 0);
        
        tmp.flush();
        return result;
    }
    
    public GreenfootImage[] getSubImgs(int x, int y, int spacing, int width, int height, int amount) {
        GreenfootImage[] result = new GreenfootImage[amount];
        for (int i = 0; i < amount; i++) {
            result[i]  = getSubImage(x + i * (width + spacing), y, width, height);
        }
        return result;
    }
    
    public GreenfootImage[] getSubImgsVariableWidth(int x, int y, int spacing, int[] widths, int height) {
        if (widths[0] != 0) {
            int[] newWidths = new int[widths.length + 1];
            System.arraycopy(widths, 0, newWidths, 1, widths.length);
            widths = newWidths;
        }
        
        GreenfootImage[] result = new GreenfootImage[widths.length - 1];
        int lastX = x - spacing;
        for (int i = 0; i < widths.length - 1; i++) {
            result[i] = getSubImage(lastX = lastX + widths[i] + spacing, y, widths[i + 1], 20);
            System.out.println(i + " x: " + lastX + ", width: " + widths[i + 1]);
        }
        
        
        return result;
    }
    
    public GreenfootImage getImage() {
        return this.image;
    }
    
    public int getWidth() {
        return this.image.getWidth();
    }
    
    public int getHeight() {
        return this.image.getHeight();
    }

    
}