import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;
import java.util.Arrays;
import java.util.ArrayList;
import java.net.Socket;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

public class LocalPlayer extends Player implements Runnable{

    private Hitbox hitbox;
    private boolean isTransparent = false;
    private boolean bombKeyWasPressedBefore;
    private int safeTime = 0;
    private int bombFuse = 180;
    private ArrayList<Direction> directions = new ArrayList<Direction>(Arrays.asList(Direction.values()));
    private String name;
    
    private final String KEY_UP;
    private final String KEY_DOWN;
    private final String KEY_RIGHT;
    private final String KEY_LEFT;
    private final String KEY_BOMB;
    
    private boolean online = false;
    
    private Socket socketToServer;
    public static ObjectOutputStream outToServer;
    public static ObjectInputStream inFromServer;
    public int ownID;

    public LocalPlayer(GreenfootImage[] images, String KEY_UP, String KEY_DOWN, String KEY_RIGHT, String KEY_LEFT, String KEY_BOMB, Socket socketToServer, String name) {
        super(images.clone());
        hitbox = new Hitbox(HITBOX_SIZE, HITBOX_SIZE, this);
        active = false;
        this.KEY_UP = KEY_UP;
        this.KEY_DOWN = KEY_DOWN;
        this.KEY_RIGHT = KEY_RIGHT;
        this.KEY_LEFT = KEY_LEFT;
        this.KEY_BOMB = KEY_BOMB;
        this.socketToServer = socketToServer;
        this.name = name;
        if (socketToServer != null)
            prepareConnection();
    }
    
    public LocalPlayer reincarnate(){
        return new LocalPlayer(images, KEY_UP, KEY_DOWN, KEY_RIGHT, KEY_LEFT, KEY_BOMB, socketToServer, name);
    }

    public LocalPlayer(GreenfootImage[] images, Socket socketToServer) {
        super(images.clone());
        hitbox = new Hitbox(HITBOX_SIZE, HITBOX_SIZE, this);
        active = false;
        this.KEY_UP = "up";
        this.KEY_DOWN = "down";
        this.KEY_RIGHT = "right";
        this.KEY_LEFT = "left";
        this.KEY_BOMB = "space";
        this.socketToServer = socketToServer;
        if (socketToServer != null)
            prepareConnection();
    }
    
    public void prepareConnection(){
            online = true;
            new Thread(this).start();
    }

    @Override
    public void firstAct() {
        System.out.println("first act!");
        getWorld().addObject(hitbox, getX(), getY());
    }

    @Override 
    public void doAct() {
        
        safeTime--;
        if (safeTime > 0 && !isTransparent){
            for (GreenfootImage img : images)
                img.setTransparency(125);
            isTransparent = true;
        }else if (isTransparent && safeTime <= 0){
            for (GreenfootImage img : images)
                img.setTransparency(255);
            isTransparent = false;
        }
        for (int i = 0; i < bombCount; i++)
            if (bombTracker[i] > 0)
                bombTracker[i]--;

        if (active) {
            //String lastKey = Greenfoot.getKey(); // getKeyPressed, not typed... TODO
            //if (lastKey != null)
            //    if (lastKey.equals(KEY_UP)) {
            //        directions.remove(Direction.UP);
            //        directions.add(0,Direction.UP);
            //    }else if (lastKey.equals(KEY_DOWN)){
            //        directions.remove(Direction.DOWN);
            //        directions.add(0,Direction.DOWN);                        
            //    }else if (lastKey.equals(KEY_RIGHT)){
            //        directions.remove(Direction.RIGHT);
            //        directions.add(0,Direction.RIGHT);                        
            //    }else if (lastKey.equals(KEY_LEFT)){
            //        directions.remove(Direction.LEFT);
            //        directions.add(0,Direction.LEFT);                        
            //    }
            //if (Greenfoot.isKeyDown("t")){
            //    System.out.println("--------");
            //    for (Direction dir : directions)
            //        System.out.println(dir.name());
            //    System.out.println("--------");
            //}

            for (Direction dir : directions) {     
                if (Greenfoot.isKeyDown(directionToString(dir))) {
                    float exactXFORESEEN = exactX + dir.getModX()*speed;
                    float exactYFORESEEN = exactY + dir.getModY()*speed;

                    hitbox.setLocation((int) exactXFORESEEN,(int) exactYFORESEEN + HITBOX_OFFSET_Y);

                    boolean isLegit = true;
                    for (Actor a : hitbox.getIntersections())
                        if (a instanceof Block || a instanceof DestroyableBlock){
                            isLegit = false;
                            break;
                        } else if (a instanceof Bomb){
                            hitbox.setLocation( (int) getX(), (int) exactY + HITBOX_OFFSET_Y);
                            List<Bomb> list = hitbox.getIntersectingBombs();
                            if (bombKick && !list.contains(a))
                                ((Bomb)a).kick(dir, 1.5f*speed, this);
                            boolean moveInBomb = false;
                            for (Bomb bomb : list)
                                if (a.equals(bomb))
                                    moveInBomb = true;
                                else{
                                    moveInBomb = false;
                                    break;
                                }
                            if (!moveInBomb){
                                isLegit = false;
                                break;
                            }
                        }
                    if (isLegit){
                        this.move(dir);
                        break;
                    }
                }
            }
            hitbox.setLocation((int) exactX, (int) exactY + HITBOX_OFFSET_Y);

            Item item = hitbox.getOneIntersectingItem();
            if (item != null)
                item.getEffectFor(this);

            if (Greenfoot.isKeyDown(KEY_BOMB) && !bombKeyWasPressedBefore && !hitbox.getIntersectingTypes().contains(Bomb.class)){
                for (int i = 0; i < bombCount; i++)
                    if (bombTracker[i] == 0){
                        bombTracker[i] = bombFuse;
                        Bomb bomb = new Bomb(bombRange, bombFuse);
                        getWorld().addObject(bomb, getX(), getY());
                        break;
                    }
                bombKeyWasPressedBefore = true;
            }else if (!Greenfoot.isKeyDown(KEY_BOMB))
                bombKeyWasPressedBefore = false;   

            if (walking) {
                dirMod++;
                setCurrentImage();
                walking = false;
            }
            
        }    
    }

    public void checkDeath(){

        if (safeTime < 0){
            lives--;

            if (lives < 0){
                getWorld().addObject(new Grave(),getX(), getY());
                active = false;
                getWorld().removeObject(hitbox);
                getWorld().removeObject(this);
            }else
                safeTime = 200;
        }
    }
    
    private String directionToString(Direction d){
        switch(d){
            case UP:
                return KEY_UP;
            case DOWN:
                return KEY_DOWN;
            case RIGHT:
                return KEY_RIGHT;
            case LEFT:
                return KEY_LEFT;
            default:
                return null;
        }
    }
    
    public void run(){
        while (true){
            try{
                Thread.sleep(500);
                System.out.println("-------\nLocalPlayer is sending a his PlayerUpdateProtocol");
                ((OnlineMap)getWorld()).sendToHost(new PlayerUpdateProtocol((int)exactX, (int)exactY, imageIndex, safeTime > 0, new Integer(ownID)));
            }catch(Exception e){
                System.out.println("Could not send playerUpdate to server");
                e.printStackTrace();
            }
        }
    }
    
    
    public String getName(){
        return name;
    }
    
}
