import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;
import java.lang.Math;

public class MenuCharacter extends MenuActors {
    
    private GreenfootImage[] frames;
    private Direction currentDir = Direction.RIGHT;
    
    private int ticksAlive;
    private int turnTicks;
    private boolean lastState = false;
    private final boolean turn;
    private final int character;
    private List<GreenfootImage[]> characterFrames;
    
    public MenuCharacter(int sizeMultiplier, boolean turn, int character){
        this.turn = turn;
        this.character = character;
        characterFrames = Resources.getFreshCharacterImages(sizeMultiplier);
        
        for (GreenfootImage[] frames : characterFrames)
            for (GreenfootImage image : frames) {
                image.scale((int) (image.getWidth() * 2.75), (int) (image.getHeight() * 2.75));
            }
        
        frames = characterFrames.get(character == -1 ? Greenfoot.getRandomNumber(characterFrames.size()) : character);
        setCurrentImage();
    }
    
    private void setCurrentImage() {
        setImage(frames[currentDir.ordinal() * 2 + (lastState ? 1 : 0)]);
        
        lastState = !lastState;
    }
    
    public void act() {
        if (ticksAlive++ % 13 != 0)
            return;
        
        MouseInfo info = Greenfoot.getMouseInfo();
        if (info != null) {
            int deltaX = info.getX() - this.getX();
            int deltaY = info.getY() - this.getY();
            
            if (Math.abs(deltaX) > Math.abs(deltaY)) {
                currentDir = deltaX > 0 ? Direction.RIGHT : Direction.LEFT;
            } else {
                currentDir = deltaY > 0 ? Direction.DOWN : Direction.UP;
            }
        }
        
        if (turn && false){
            if (turnTicks % (16) == 0)
                currentDir = Direction.DOWN;
            else if (turnTicks % (12) == 0)
                currentDir = Direction.RIGHT;
            else if (turnTicks % (8) == 0)
                currentDir = Direction.UP;
            else if (turnTicks % (4) == 0)
                currentDir = Direction.LEFT;
            turnTicks %= 16;
            turnTicks++;
        }
        setCurrentImage();
    }
    
    public void setCharacter(int character){
        frames = characterFrames.get(character);
        
        //setCurrentImage();
    }
}
