import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.ArrayList;
import java.util.List;

/**
 * Write a description of class OfflineMap here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class OfflineMap extends Map
{
    private long lastToggle = System.currentTimeMillis();

    private List<Player> players = new ArrayList<Player>();
    private Countdown countdown = new Countdown();
    private boolean gameIsActive = false;
    private final MapTemplate mapTemplate;
    private  int MAP_OFFSET_Y = 0;
    private boolean localMultiAndSeveral;
    private boolean firstActHasWon = true;
    private Menupanel wonMessage;

    //Map.maps.get("Forest")
    public OfflineMap() {
        //test
        this(Map.maps.get("Original Bomberman"), 1);
    }

    public OfflineMap(MapTemplate mapTemplate, List<Player> players)
    {
        super(mapTemplate.panelWidth ,(players.size()-1 + 0.5) * (120 + 6) + 4 +60 > mapTemplate.panelHeight ? (int)((players.size()-1 + 0.5) * (120 + 6) + 4 +60+7) : mapTemplate.panelHeight, 1, false);

        this.mapTemplate = mapTemplate;
        
        MAP_OFFSET_Y = (getHeight()-mapTemplate.panelHeight)/2;

        this.players = players;

        for (int i = 0; i < players.size(); i++) {
            PlayerStats stats = new PlayerStats(players.get(i));
            addObject(stats, 8 + 1/2. * stats.getImage().getWidth(), (i + 0.5) * (stats.getImage().getHeight() + 6) + 4);
        }
        
        if (players.size() > 1)
            localMultiAndSeveral = true;
        setUpWorld();
        Greenfoot.setSpeed(50);
    }
    
    public OfflineMap(MapTemplate mapTemplate, int playerIndex)
    {
        super(mapTemplate.panelWidth , mapTemplate.panelHeight, 1, false);

        this.mapTemplate = mapTemplate;

        players.add(new LocalPlayer(Resources.characters.get(playerIndex), null));

        for (int i = 0; i < players.size(); i++) {
            PlayerStats stats = new PlayerStats(players.get(i));
            addObject(stats, 8 + 1/2. * stats.getImage().getWidth(), (i + 0.5) * (stats.getImage().getHeight() + 6) + 4);
        }

        setUpWorld();
        Greenfoot.setSpeed(50);
    }

    public void setUpWorld(){

        if (mapTemplate.style == "Forest"){
           GreenfootImage img = new GreenfootImage(Resources.background_menu);
           img.scale(getWidth(), getHeight());
           setBackground(img);
        }
        else if (mapTemplate.style == "Original Bomberman")
            setBackground(new GreenfootImage("original_bomberman_background.png"));
        else if (mapTemplate.style == "Circle of death")
            setBackground(new GreenfootImage("circle_background.png"));
        else if (mapTemplate.style == "1v1 me")
            setBackground(new GreenfootImage("BBC_background3.png"));
        else if (mapTemplate.style == "Squares of death")
            setBackground(new GreenfootImage("BBC_background2.png"));
        else if (mapTemplate.style == "Labyrinth")
            setBackground(new GreenfootImage("BBC_background1.png"));
        else if (mapTemplate.style == "Close Quarter Combat")
            setBackground(new GreenfootImage("BBC_background4.png"));

        int widthOfLevel = 0;
        for (int i = 0; i < mapTemplate.levelTemplate.length; i++)
            if (mapTemplate.levelTemplate[i].length > widthOfLevel)
                widthOfLevel = mapTemplate.levelTemplate[i].length;
        offset = mapTemplate.panelWidth-widthOfLevel*32+16;

        if (players.size() == 0)
            System.out.println("keine Spieler?");

        for (int y = 0, playerindex = 0; y < mapTemplate.levelTemplate.length; y++)
            for (int x = 0; x < mapTemplate.levelTemplate[y].length; x++)
                if (mapTemplate.levelTemplate[y][x] == 'b'){
                    addObject(new Block(mapTemplate.style), x * 32 + offset,y * 32 + 16+MAP_OFFSET_Y);
                }else if (mapTemplate.levelTemplate[y][x] == 'd'){
                    addObject(new DestroyableBlock(mapTemplate.style), x * 32 + offset,y * 32 + 16+MAP_OFFSET_Y);
                }else if (mapTemplate.levelTemplate[y][x] == 'p'){
                    if (!(playerindex + 1 > players.size())){
                        addObject(players.get(playerindex), x * 32 + offset,y * 32 + 16+MAP_OFFSET_Y);
                        playerindex++;
                    }
                }

        addObject(new GUIBackground(offset-16,mapTemplate.panelHeight+2*MAP_OFFSET_Y),offset/2-8,mapTemplate.panelHeight/2+MAP_OFFSET_Y);
        addObject(countdown, getWidth()/2, getHeight()/2);
    }


    public void addObject(Actor object, float x, float y) {
        super.addObject(object, (int) x, (int) y);
    }

    public void addObject(Actor object, double x, double y) {
        super.addObject(object, (int) x, (int) y);
        System.out.println("x: " + x + ", y: " + y);
    }

    public void act(){

        if (countdown.hasFinished() && !gameIsActive){
            gameIsActive = true;

            for (Player p : players)
                p.setActive(true);

            removeObject(countdown);

            //Sounds.get("csgo_main_8bit", 45).playLoop();
        }

        if (Greenfoot.isKeyDown("h") && lastToggle + 100 < System.currentTimeMillis()) {
            lastToggle = System.currentTimeMillis();
            hitboxesVisible = !hitboxesVisible;
        }
        
        if (localMultiAndSeveral && onlyOneAlive())
        {
            if(firstActHasWon){
                firstActHasWon = false;
                wonMessage = new Menupanel(new String[]{getFirstActivePlayer().getName()+" has won!","","Do you want","to play again?","Yes","No"}, 6, false, 50);
                addObject(wonMessage, getWidth()/2, getHeight()/2);
            }
            boolean overSomething = false;
            if (wonMessage.hoverOverLine(4)){
                wonMessage.selectLine(4);
                overSomething = true;
                if (Greenfoot.mouseClicked(null)){
                    Greenfoot.setWorld(new Local(mapTemplate, players.size()));
                }
            }
            if (wonMessage.hoverOverLine(5)){
                wonMessage.selectLine(5);
                overSomething = true;
                if (Greenfoot.mouseClicked(null)){
                    Greenfoot.setWorld(new Menu());
                }
            }
            if (!overSomething)
            wonMessage.selectLine(-1);
        }
    }
    
    public boolean onlyOneAlive(){
        int i = 0;
        for (Player p : players)
            if (p.isActive())
                i++;
        return i == 1;
    }
    
    public LocalPlayer getFirstActivePlayer(){
        for (Player p : players)
            if (p.isActive())
                return (LocalPlayer)p;
        return null;
    }
}
