import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.ArrayList;
import java.util.List;
import java.net.Socket;
import java.util.concurrent.ConcurrentHashMap;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * Write a description of class OnlineMap here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class OnlineMap extends Map implements Runnable
{
    private ConcurrentHashMap<Integer, Actor> actorsOnMap = new ConcurrentHashMap<Integer, Actor>();
    private int currentID = 0; // should be incremented after every call
    
    private long lastToggle = System.currentTimeMillis();
    private boolean hitboxesVisible = false;

    private List<Player> players = new ArrayList<Player>();
    private Countdown countdown = new Countdown();
    private boolean gameIsActive = false;
    private final MapTemplate mapTemplate;
    private  int MAP_OFFSET_Y = 0;
    
    private LocalPlayer localPlayer;

    private final Socket server;
    private final ObjectInputStream ois;
    private final ObjectOutputStream oos;

    static {
        new Resources().run();
    }

    public OnlineMap(MapTemplate mapTemplate, int playercount, int ownPosition, int chosenCharacter, Socket server, ObjectInputStream ois, ObjectOutputStream oos)
    {
        super(mapTemplate.panelWidth ,(playercount-1 + 0.5) * (120 + 6) + 4 +60 > mapTemplate.panelHeight ? (int)((playercount-1 + 0.5) * (120 + 6) + 4 +60+7) : mapTemplate.panelHeight, 1, false);

        this.mapTemplate = mapTemplate;
        this.server = server;
        this.ois = ois;
        this.oos = oos;
        
        MAP_OFFSET_Y = (getHeight()-mapTemplate.panelHeight)/2;
        
        setPaintOrder(new Class[] { Hitbox.class, Countdown.class, PlayerStats.class, GUIBackground.class, Explosion.SubExplosion.class, Bomb.class, Player.class, Grave.class, Block.class, DestroyableBlock.class });
        setActOrder(new Class[] { Map.class, Player.class, Bomb.class, Explosion.class});

        setUpWorld(playercount, ownPosition, chosenCharacter);
        
        //for (int i = 0; i < playercount; i++) {
          //  PlayerStats stats = new PlayerStats(players.get(i));
          //  addObject(stats, 8 + 1/2. * stats.getImage().getWidth(), (i + 0.5) * (stats.getImage().getHeight() + 6) + 4);
        //}
        
        Greenfoot.setSpeed(50);
        
        new Thread(this).start();
    }

    public void setUpWorld(int playercount, int ownPosition, int chosenCharacter){

        if (mapTemplate.style == "Forest"){
           GreenfootImage img = new GreenfootImage(Resources.background_menu);
           img.scale(getWidth(), getHeight());
           setBackground(img);
        }
        else if (mapTemplate.style == "Original Bomberman")
            setBackground(new GreenfootImage("original_bomberman_background.png"));
        else if (mapTemplate.style == "Circle of death")
            setBackground(new GreenfootImage("circle_background.png"));    

        int widthOfLevel = 0;
        for (int i = 0; i < mapTemplate.levelTemplate.length; i++)
            if (mapTemplate.levelTemplate[i].length > widthOfLevel)
                widthOfLevel = mapTemplate.levelTemplate[i].length;
        offset = mapTemplate.panelWidth-widthOfLevel*32+16;

        if (playercount == 0)
            System.out.println("keine Spieler?");

        for (int y = 0, playerindex = 0; y < mapTemplate.levelTemplate.length; y++)
            for (int x = 0; x < mapTemplate.levelTemplate[y].length; x++){
                if (mapTemplate.levelTemplate[y][x] == 'b'){
                    Block b = new Block(mapTemplate.style);
                    actorsOnMap.put(new Integer(currentID),b);
                    addObject(b, x * 32 + offset,y * 32 + 16+MAP_OFFSET_Y);
                }else if (mapTemplate.levelTemplate[y][x] == 'd'){
                    DestroyableBlock b = new DestroyableBlock(mapTemplate.style);
                    actorsOnMap.put(new Integer(currentID),b);
                    addObject(b, x * 32 + offset,y * 32 + 16+MAP_OFFSET_Y);
                }else if (mapTemplate.levelTemplate[y][x] == 'p'){
                    if (playerindex < playercount){
                        if (playerindex == ownPosition){
                            localPlayer = new LocalPlayer(Resources.characters.get(chosenCharacter), server);
                            localPlayer.ownID = currentID;
                            actorsOnMap.put(new Integer(currentID), localPlayer);
                            addObject(localPlayer, x * 32 + offset,y * 32 + 16 +MAP_OFFSET_Y);
                        }else{
                            OnlinePlayer p = new OnlinePlayer(3);
                            actorsOnMap.put(new Integer(currentID), p);//TODO
                            addObject(p, x * 32 + offset,y * 32 + 16 +MAP_OFFSET_Y);
                        }
                        playerindex++;
                    }
                }
                currentID++;
            }
            
        addObject(new GUIBackground(offset-16,mapTemplate.panelHeight+2*MAP_OFFSET_Y),offset/2-8,mapTemplate.panelHeight/2+MAP_OFFSET_Y);
        addObject(countdown, getWidth()/2, getHeight()/2);
    }


    public void addObject(Actor object, float x, float y) {
        super.addObject(object, (int) x, (int) y);
    }

    public void addObject(Actor object, double x, double y) {
        super.addObject(object, (int) x, (int) y);
        System.out.println("x: " + x + ", y: " + y);
    }

    public void act(){

        if (countdown.hasFinished() && !gameIsActive){
            gameIsActive = true;

            localPlayer.setActive(true);

            removeObject(countdown);

            //Sounds.get("csgo_main_8bit", 45).playLoop();
        }

        if (Greenfoot.isKeyDown("h") && lastToggle + 100 < System.currentTimeMillis()) {
            lastToggle = System.currentTimeMillis();
            hitboxesVisible = !hitboxesVisible;
        }
    }
    
    public synchronized void sendToHost(Object o){
        try{
            System.out.println("OnlineMap is sending a protocol");
            oos.writeObject(o);
            oos.flush();
        }catch(Exception e){e.printStackTrace();}
    }
    
    public void run(){
        Object recievedObject;
        try{
            while((recievedObject = ois.readObject()) != null){
                System.out.println("OnlineMap has recieved a protocol");
                if (recievedObject instanceof PlayerUpdateProtocol){
                    PlayerUpdateProtocol extract = (PlayerUpdateProtocol) recievedObject;
                    OnlinePlayer actor = (OnlinePlayer) actorsOnMap.get(extract.playerID);
                    if (actor != null){
                        System.out.println("the onlinePlayer's location is going to be changed");
                        actor.setLocation(extract.posX, extract.posY);
                        actor.setImageWithIndex(extract.animationImage, extract.transparent);
                    }
                }else if (recievedObject instanceof NewBombProtocol){
                    
                }else if (recievedObject instanceof BombUpdateProtocol){
                    
                }else if (recievedObject instanceof BlockDestroyedProtocol){
                    
                }else if (recievedObject instanceof PlayerDiedProtocol){
                    
                }

            }
        }catch(Exception e){e.printStackTrace();}
    }
}
