import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

public abstract class Player extends ExactLocated {

    protected final static int HITBOX_OFFSET_Y = 4; // offset of the hitbox to the player's feet
    protected final static int HITBOX_SIZE = 24; // size of the player's hitbox
    protected final static int START_LIVES = 2;
    protected final static float INITIAL_SPEED = 1.4f;

    protected GreenfootImage[] images;
    protected Direction direction; // direction the player is facing
    protected int dirMod; // for animation purpose
    protected boolean walking; // if in next act the animation should be played

    protected boolean active; // whether player can move etc.

    protected float speed = INITIAL_SPEED; // walkingspeed: 1f == 1 pixel per act
    protected int[] bombTracker = new int[30];

    // Attributes - upgradable through items
    protected boolean bombKick = false;
    protected int bombRange = 1; // max range a bomb has
    protected int bombCount = 1; // max amount of bombs placed simultaneously
    protected int lives = START_LIVES;
    protected int imageIndex;

    public Player(GreenfootImage[] images) {
        if (images.length != 8) throw new IllegalArgumentException("Image array has to contain 8 elements");
        this.images = images;

        this.direction = Direction.DOWN;

        setCurrentImage();
    }

    protected void setCurrentImage() {
        this.imageIndex = direction.ordinal() * 2 + (dirMod / 4) % 2;
        setImage(images[imageIndex]);
    }

    public void turn(Direction direction) {
        this.direction = direction;
        setCurrentImage();
    }

    public void move(Direction direction) {
        this.direction = direction;

        exactX += direction.getModX()*speed;
        exactY += direction.getModY()*speed;

        this.setLocation((int) exactX, (int) exactY);
        this.walking = true;
    }

    public boolean isActive() {
        return active;
    }

    public void setActive(boolean active) {
        this.active = active;
    }

    public float getSpeed(){
        return speed;
    }

    public void setSpeed(float speed){
        this.speed = speed;
    }

    public int getBombRange() {
        return this.bombRange;
    }

    public void setBombRange(int i){
        this.bombRange = i;
    }

    public int getBombCount(){
        return this.bombCount;
    }

    public void setBombCount(int i){
        this.bombCount = i;
    }

    public boolean hasBombKick(){
        return this.bombKick;
    }

    public void setBombKick(boolean i){
        this.bombKick = i;
    }
    
    public int getLives() {
        return this.lives;
    }
    
    public GreenfootImage[] getImages() {
        return this.images;
    }
    
    public int getCurrentlyPlacedBombCount() {
        int result = 0;
        for (int i = 0; i < bombCount; i++) {
            if (bombTracker[i] > 0) result++;
        }
        return result;
        
    }
}
