import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.*;
import greenfoot.Color;

public class PlayerStats extends GUI {
    private static final GreenfootImage[] bombFrames = Bomb.getFreshFrames();
    private static final GreenfootImage sneaker = new GreenfootImage("sneaker.png");
    private static final GreenfootImage range = new GreenfootImage("range.png");
    private static final GreenfootImage heart = new GreenfootImage("heart.png");
    private static final GreenfootImage grave = new GreenfootImage("grave.png");
    private static final GreenfootImage bombKick = new GreenfootImage("bomb_kick.png");

    static {
        for (GreenfootImage bomb : bombFrames)
            bomb.scale(24, (int) (24./bomb.getWidth() * bomb.getHeight()));
        
        sneaker.scale(32, 32);
        range.scale(32, 32);
        heart.scale(24, (int) (24./heart.getWidth() * heart.getHeight()));
        bombKick.scale(24, (int) (24./bombKick.getWidth() * bombKick.getHeight()));
        
        grave.scale((int) (32./grave.getHeight() * grave.getWidth()), 32);
    }

    private GreenfootImage image, currentBurningBomb;
    private Player player;

    public PlayerStats() { this(null) ;}

    public PlayerStats(Player player) {
        this.player = player;
        this.image = new GreenfootImage(200, 120);

        draw();
    }

    private void draw() {
//        String name = "Name"; // TODO

        image.setColor(Color.BLACK);
        image.fill();
        image.setColor(Color.WHITE);
        image.fillRect(5, 5, image.getWidth() - 10, image.getHeight() - 10);
        
        GreenfootImage character = player.getLives() < 0 ? grave : player.getImages()[Direction.RIGHT.ordinal() * 2];
        image.drawImage(character, 8, 8);
        
        image.setColor(Color.BLACK);
        //image.setFont(new Font(Font.MONOSPACED, Font.BOLD, 16));
        image.setFont(new greenfoot.Font(java.awt.Font.MONOSPACED, true, false, 16));
//        image.drawString(name, 8 + 32 + 3, 8 + 16 + 8);

        int speed = (int) ((player.getSpeed() - Player.INITIAL_SPEED) / Item.SPEED_UPGRADE_ADDITION + 1);

        for (int i = 0; i < speed; i++) {
            image.drawImage(player.hasBombKick() ? bombKick : sneaker, 3 + i * 18, 32);
        }

        int currentlyPlacedBombs = player.getCurrentlyPlacedBombCount();
        
        if (ticksAlive % 17 == 0) currentBurningBomb = bombFrames[ticksAlive % 2 + 1];
        
        for (int i = 0; i < player.getBombCount(); i++) {
            image.drawImage(i < currentlyPlacedBombs ? currentBurningBomb : bombFrames[0], 8 + i * 18, 54);
        }

        for (int i = 0; i < player.getBombRange(); i++) {
            image.drawImage(range, 3 + i * 18, 74);
        }

        for (int i = 0; i < player.getLives()+1; i++) {
            image.drawImage(heart, image.getWidth() - 20 * Player.START_LIVES + i * 16-20, 4);
        }
        
        setImage(image);

    }

    public void doAct() {
        draw();
    }    
}
